@@ -1,5 +1,5 @@
 /*
- * $Id: afp_config.h,v 1.3 2001/12/03 05:03:38 jmarcus Exp $
+ * $Id: afp_config.h,v 1.5 2009-10-13 22:55:36 didg Exp $
  */
 
 #ifndef AFPD_CONFIG_H
@@ -17,15 +17,15 @@ typedef struct AFPConfig {
     AFPObj obj;
     int fd, statuslen;
     unsigned char *optcount;
-    char status[ATP_MAXDATA];
+    char status[1400];
     const void *defoptions, *signature;
-    int (*server_start) __P((struct AFPConfig *, struct AFPConfig *,
-                             server_child *));
-    void (*server_cleanup) __P((const struct AFPConfig *));
+    int (*server_start) (struct AFPConfig *, struct AFPConfig *,
+                             server_child *);
+    void (*server_cleanup) (const struct AFPConfig *);
     struct AFPConfig *next;
 } AFPConfig;
 
-extern AFPConfig *configinit __P((struct afp_options *));
-extern void configfree __P((AFPConfig *, const AFPConfig *));
+extern AFPConfig *configinit (struct afp_options *);
+extern void configfree (AFPConfig *, const AFPConfig *);
 
 #endif
