@@ -1,6 +1,4 @@
 /*
- * $Id: main.c,v 1.26 2009-10-14 02:24:05 didg Exp $
- *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
  * All Rights Reserved.  See COPYRIGHT.
  */
@@ -101,7 +99,7 @@ static void afp_goaway(int sig)
     dsi_kill(sig);
     switch( sig ) {
     case SIGTERM :
-        LOG(log_info, logtype_afpd, "shutting down on signal %d", sig );
+        LOG(log_note, logtype_afpd, "AFP Server shutting down on SIGTERM");
         break;
     case SIGUSR1 :
     case SIGHUP :
@@ -188,11 +186,6 @@ int main(int ac, char **av)
         exit(0);
     }
 
-#if 0
-    /* Register CNID  */
-    cnid_init();
-#endif
-
     /* install child handler for asp and dsi. we do this before afp_goaway
      * as afp_goaway references stuff from here. 
      * XXX: this should really be setup after the initial connections. */
@@ -201,16 +194,21 @@ int main(int ac, char **av)
         LOG(log_error, logtype_afpd, "main: server_child alloc: %s", strerror(errno) );
         afp_exit(EXITERR_SYS);
     }
-    
+
+    memset(&sv, 0, sizeof(sv));    
 #ifdef AFP3x
     /* linux at least up to 2.4.22 send a SIGXFZ for vfat fs,
        even if the file is open with O_LARGEFILE ! */
 #ifdef SIGXFSZ
-    signal(SIGXFSZ , SIG_IGN); 
+    sv.sa_handler = SIG_IGN;
+    sigemptyset( &sv.sa_mask );
+    if (sigaction(SIGXFSZ, &sv, NULL ) < 0 ) {
+        LOG(log_error, logtype_afpd, "main: sigaction: %s", strerror(errno) );
+        afp_exit(EXITERR_SYS);
+    }
+#endif
 #endif
-#endif    
     
-    memset(&sv, 0, sizeof(sv));
     sv.sa_handler = child_handler;
     sigemptyset( &sv.sa_mask );
     sigaddset(&sv.sa_mask, SIGALRM);
