@@ -1,5 +1,5 @@
 /*
- * $Id: queries.c,v 1.16.2.1.2.1 2004/06/09 01:25:54 bfernhomberg Exp $
+ * $Id: queries.c,v 1.22 2009-10-13 22:55:37 didg Exp $
  *
  * Copyright (c) 1990,1994 Regents of The University of Michigan.
  * All Rights Reserved.  See COPYRIGHT.
@@ -57,8 +57,7 @@ int cq_rbilogin( struct papfile *, struct papfile * );
 
 
 
-int cq_default( in, out )
-    struct papfile	*in, *out;
+int cq_default( struct papfile *in, struct papfile *out)
 {
     char		*start, *stop, *p;
     int			linelength, crlflength;
@@ -71,6 +70,9 @@ int cq_default( in, out )
 
 	case -1 :
 	    return( CH_MORE );
+
+        case -2 :
+            return( CH_ERROR );
 	}
 
 	stop = start+linelength;
@@ -91,9 +93,11 @@ int cq_default( in, out )
 			break;
 		    }
 		}
-		p++;
-		while ( *p == ' ' ) {
+		if (p < stop) {
 		    p++;
+		    while ( *p == ' ' ) {
+		        p++;
+                    }
 		}
 
 		append( out, p, stop - p + crlflength );
@@ -113,8 +117,7 @@ char	*LoginFailed = "LoginFailed\n";
 
 #define h2b(x)	(isdigit((x))?(x)-'0':(isupper((x))?(x)-'A':(x)-'a')+10)
 
-int cq_k4login( in, out )
-    struct papfile	*in, *out;
+int cq_k4login( struct papfile *in, struct papfile *out)
 {
     char		*start, *p;
     int			linelength, crlflength;
@@ -130,6 +133,9 @@ int cq_k4login( in, out )
 
     case -1 :
 	return( CH_MORE );
+
+    case -2 :
+        return( CH_ERROR );
     }
 
     p = start + strlen( comment->c_begin );
@@ -139,6 +145,7 @@ int cq_k4login( in, out )
 
     bzero( &tkt, sizeof( tkt ));
     stop = start+linelength;
+    /* FIXME */
     for ( i = 0, t = tkt.dat; p < stop; p += 2, t++, i++ ) {
 	*t = ( h2b( (unsigned char)*p ) << 4 ) +
 		h2b( (unsigned char)*( p + 1 ));
@@ -166,8 +173,7 @@ int cq_k4login( in, out )
 
 char	*uameth = "UMICHKerberosIV\n*\n";
 
-int cq_uameth( in, out )
-    struct papfile	*in, *out;
+int cq_uameth( struct papfile *in, struct papfile *out)
 {
     char		*start;
     int			linelength, crlflength;
@@ -180,6 +186,9 @@ int cq_uameth( in, out )
 
 	case -1 :
 	    return( CH_MORE );
+
+        case -2 :
+            return( CH_ERROR );
 	}
 
 	if ( comgetflags() == 0 ) {	/* start */
@@ -204,8 +213,7 @@ int cq_uameth( in, out )
 }
 #endif /* KRB */
 
-int gq_true( out )
-    struct papfile	*out;
+int gq_true( struct papfile *out)
 {
     if ( printer->p_flags & P_SPOOLED ) {
 	append( out, "true\n", 5 );
@@ -215,8 +223,7 @@ int gq_true( out )
     }
 }
 
-int gq_pagecost( out )
-    struct papfile	*out;
+int gq_pagecost( struct papfile *out)
 {
     char		cost[ 60 ];
 
@@ -238,8 +245,7 @@ int gq_pagecost( out )
 }
 
 #ifdef ABS_PRINT
-int gq_balance( out )
-    struct papfile	*out;
+int gq_balance( struct papfile *out)
 {
     char		balance[ 60 ];
 
@@ -259,8 +265,7 @@ int gq_balance( out )
 
 static const char *spoolerid = "(PAPD Spooler) 1.0 (" VERSION ")\n";
 
-int gq_rbispoolerid( out )
-    struct papfile	*out;
+int gq_rbispoolerid( struct papfile *out)
 {
     append( out, spoolerid, strlen( spoolerid ));
     return(0);
@@ -274,8 +279,7 @@ int gq_rbispoolerid( out )
 
 static const char *nouams = "*\n";
 
-int gq_rbiuamlist( out )
-    struct papfile      *out;
+int gq_rbiuamlist( struct papfile *out)
 {
     char uamnames[128] = "\0";
 
@@ -307,11 +311,10 @@ struct genquery {
     { "ADOIsBinaryOK?", gq_true },
     { "UMICHListQueue", gq_true },
     { "UMICHDeleteJob", gq_true },
-    { NULL },
+    { NULL, NULL },
 };
 
-int cq_query( in, out )
-    struct papfile	*in, *out;
+int cq_query( struct papfile *in, struct papfile *out)
 {
     char		*start, *stop, *p, *q;
     int			linelength, crlflength;
@@ -326,6 +329,9 @@ int cq_query( in, out )
 
 	case -1 :
 	    return( CH_MORE );
+
+        case -2 :
+            return( CH_ERROR );
 	}
 
 	stop = start+linelength;
@@ -339,7 +345,7 @@ int cq_query( in, out )
 		}
 	    }
 
-	    for ( p++; p < stop; p++ ) {
+	    if (p < stop) for ( p++; p < stop; p++ ) {
 		if ( *p != ' ' && *p != '\t' ) {
 		    break;
 		}
@@ -352,7 +358,7 @@ int cq_query( in, out )
 	    }
 
 	    for ( gq = genqueries; gq->gq_name; gq++ ) {
-		if (( strlen( gq->gq_name ) == q - p ) &&
+		if (( strlen( gq->gq_name ) == (size_t)(q - p) ) &&
 			( strncmp( gq->gq_name, p, q - p ) == 0 )) {
 		    break;
 		}
@@ -377,23 +383,23 @@ int cq_query( in, out )
     }
 }
 
-void cq_font_answer( start, stop, out )
-    char		*start, *stop;
-    struct papfile	*out;
+void cq_font_answer( char *start, char *stop, struct papfile *out)
 {
     char		*p, *q, buf[ 256 ];
     struct ppd_font	*pfo;
 
     p = start;
     while ( p < stop ) {
+        unsigned int count = 0;
 	while (( *p == ' ' || *p == '\t' ) && p < stop ) {
 	    p++;
 	}
 
 	q = buf;
 	while ( *p != ' ' && *p != '\t' &&
-		*p != '\n' && *p != '\r' && p < stop ) {
+		*p != '\n' && *p != '\r' && p < stop && count < sizeof(buf)) {
 	    *q++ = *p++;
+	    count++;
 	}
 
 	if ( q != buf ) {
@@ -414,8 +420,7 @@ void cq_font_answer( start, stop, out )
     return;
 }
 
-int cq_font( in, out )
-    struct papfile	*in, *out;
+int cq_font(struct papfile *in, struct papfile *out)
 {
     char		*start, *stop, *p;
     int			linelength, crlflength;
@@ -428,6 +433,9 @@ int cq_font( in, out )
 
 	case -1 :
 	    return( CH_MORE );
+
+        case -2 :
+            return( CH_ERROR );
 	}
 
 	stop = start + linelength;
@@ -440,7 +448,8 @@ int cq_font( in, out )
 		    break;
 		}
 	    }
-	    p++;
+	    if (p < stop)
+	        p++;
 
 	    cq_font_answer( p, stop, out );
 	} else {
@@ -453,7 +462,8 @@ int cq_font( in, out )
 			break;
 		    }
 		}
-		p++;
+		if (p < stop)
+		    p++;
 
 		cq_font_answer( p, stop, out );
 	    } else {
@@ -471,8 +481,7 @@ int cq_font( in, out )
     }
 }
 
-int cq_feature( in, out )
-    struct papfile	*in, *out;
+int cq_feature( struct papfile *in, struct papfile *out)
 {
     char		*start, *stop, *p;
     int			linelength, crlflength;
@@ -486,6 +495,9 @@ int cq_feature( in, out )
 
 	case -1 :
 	    return( CH_MORE );
+
+        case -2 :
+            return( CH_ERROR );
 	}
 
 	stop = start + linelength;
@@ -499,7 +511,8 @@ int cq_feature( in, out )
 		    break;
 		}
 	    }
-	    p++;
+	    if (p < stop)
+	        p++;
 	    while ( *p == ' ' ) {
 		p++;
 	    }
@@ -529,8 +542,7 @@ int cq_feature( in, out )
 static const char	*psver = "*PSVersion\n";
 static const char	*prod = "*Product\n";
 
-int cq_printer( in, out )
-    struct papfile	*in, *out;
+int cq_printer(struct papfile *in, struct papfile *out)
 {
     char		*start, *p;
     int			linelength, crlflength;
@@ -544,6 +556,9 @@ int cq_printer( in, out )
 
 	case -1 :
 	    return( CH_MORE );
+
+        case -2 :
+            return( CH_ERROR );
 	}
 
 	if ( comgetflags() == 0 ) {
@@ -607,8 +622,7 @@ int cq_printer( in, out )
 static const char	*rmjobfailed = "Failed\n";
 static const char	*rmjobok = "Ok\n";
 
-int cq_rmjob( in, out )
-    struct papfile	*in, *out;
+int cq_rmjob( struct papfile *in, struct papfile *out)
 {
     char		*start, *stop, *p;
     int			linelength, crlflength;
@@ -620,6 +634,9 @@ int cq_rmjob( in, out )
 
     case -1 :
 	return( CH_MORE );
+
+    case -2 :
+        return( CH_ERROR );
     }
 
     stop = start + linelength;
@@ -648,8 +665,7 @@ int cq_rmjob( in, out )
     return( CH_DONE );
 }
 
-int cq_listq( in, out )
-    struct papfile	*in, *out;
+int cq_listq( struct papfile *in, struct papfile *out)
 {
     char		*start;
     int			linelength, crlflength;
@@ -660,6 +676,9 @@ int cq_listq( in, out )
 
     case -1 :
 	return( CH_MORE );
+
+    case -2 :
+        return( CH_ERROR );
     }
 
     if ( lp_queue( out )) {
@@ -685,14 +704,13 @@ SecurityViolation: Unknown user, incorrect password or log on is \
 disabled ]%%\r%%[Flushing: rest of job (to end-of-file) will be \
 ignored ]%%\r";
 
-int cq_rbilogin( in, out )
-    struct papfile      *in, *out;
+int cq_rbilogin( struct papfile *in, struct papfile *out)
 {
     char        	*start, *stop, *p, *begin;
     int			linelength, crlflength;
     char        	username[UAM_USERNAMELEN + 1] = "\0";
     struct papd_comment	*comment = compeek();
-    char		uamtype[20] = "\0";
+    char		uamtype[20];
 
     for (;;) {
         switch ( markline( in, &start, &linelength, &crlflength )) {
@@ -701,6 +719,9 @@ int cq_rbilogin( in, out )
 
         case -1 :
             return( CH_MORE );
+
+        case -2 :
+            return( CH_ERROR );
         }
 
 	stop = start + linelength;
@@ -709,13 +730,16 @@ int cq_rbilogin( in, out )
 	    begin = start + strlen(comment->c_begin);
 	    p = begin;
 
-	    while (*p != ' ') {
+	    while (*p != ' ' && p < stop) {
 		p++;
 	    }
 
-	    strncat(uamtype, begin, p - begin);
+	    memset(uamtype, 0, sizeof(uamtype));
+	    if ((size_t)(p -begin) <= sizeof(uamtype) -1) {
+	        strncpy(uamtype, begin, p - begin);
+            }
 
-	    if ((papd_uam = auth_uamfind(UAM_SERVER_PRINTAUTH,
+	    if ( !*uamtype || (papd_uam = auth_uamfind(UAM_SERVER_PRINTAUTH,
 				uamtype, strlen(uamtype))) == NULL) {
 		LOG(log_info, logtype_papd, "Could not find uam: %s", uamtype);
 		append(out, rbiloginbad, strlen(rbiloginbad));
@@ -749,18 +773,18 @@ int cq_rbilogin( in, out )
  */
 struct papd_comment	queries[] = {
 #ifdef KRB
-    { "%%Login: UMICHKerberosIV", 0,			cq_k4login,	0 },
-    { "%%?BeginUAMethodsQuery",	"%%?EndUAMethodsQuery:", cq_uameth, C_FULL },
+    { "%%Login: UMICHKerberosIV", NULL,			cq_k4login,	0 },
+    { "%%?BeginUAMethodsQuery",	"%%?EndUAMethodsQuery:", cq_uameth,C_FULL },
 #endif /* KRB */
 #ifndef HAVE_CUPS
-    { "%UMICHListQueue", 0,				cq_listq, C_FULL },
-    { "%UMICHDeleteJob", 0,				cq_rmjob,	0 },
+    { "%UMICHListQueue",	NULL,			cq_listq,  C_FULL },
+    { "%UMICHDeleteJob",	NULL,			cq_rmjob,	0 },
 #endif /* HAVE_CUPS */
     { "%%?BeginQuery: RBILogin ", "%%?EndQuery",	cq_rbilogin,	0 },
     { "%%?BeginQuery",		"%%?EndQuery",		cq_query,	0 },
     { "%%?BeginFeatureQuery",	"%%?EndFeatureQuery",	cq_feature,	0 },
     { "%%?BeginFontQuery",	"%%?EndFontQuery",	cq_font,	0 },
-    { "%%?BeginPrinterQuery",	"%%?EndPrinterQuery",	cq_printer, C_FULL },
+    { "%%?BeginPrinterQuery",	"%%?EndPrinterQuery",	cq_printer,C_FULL },
     { "%%?Begin",		"%%?End",		cq_default,	0 },
-    { 0 },
+    { NULL,			NULL,			NULL,		0 },
 };
